@@ -1,7 +1,8 @@
 from pymatgen.electronic_structure.core import Spin, Orbital
 from pymatgen.io.vasp.outputs import BSVasprun, Eigenval
 from pymatgen.io.vasp.inputs import Kpoints, Poscar, Incar
-from .unfold.unfold import unfold, make_kpath, removeDuplicateKpoints
+from pymatgen.symmetry.bandstructure import HighSymmKpath
+from vaspvis.unfold import unfold, make_kpath, removeDuplicateKpoints
 from pymatgen.core.periodic_table import Element
 from pyprocar.utilsprocar import UtilsProcar
 from pyprocar.procarparser import ProcarParser
@@ -27,40 +28,45 @@ class Band:
 
     Parameters:
         folder (str): This is the folder that contains the VASP files
-        projected (bool): Determined wheter of not to parte the projected
-            eigenvalues from the vasprun.xml file. Making this true
+        projected (bool): Determines whether of not to parse the projected
+            eigenvalues from the PROCAR file. Making this true
             increases the computational time, so only use if a projected
             band structure is required.
-        hse (bool): Determines if the KPOINTS file is in the form of HSE
-            or not. Only make true if the band structure was calculated
-            using a hybrid functional.
         spin (str): Choose which spin direction to parse. ('up' or 'down')
         kpath (str): High symmetry k-point path of band structure calculation
-            Due to the nature of the KPOINTS file for HSE calculations this
+            Due to the nature of the KPOINTS file for unfolded calculations this
             information is a required input for proper labeling of the figure
-            for HSE calculations. This information is extracted from the KPOINTS
-            files for non-HSE calculations. (G is automaticall converted to \\Gamma)
-        n (int): Number of points between each high symmetry points.
-            This is also only required for HSE calculations. This number should be 
+            for unfolded calculations. This information is extracted from the KPOINTS
+            files for non-unfolded calculations. (G is automaticall converted to \\Gamma)
+        n (int): Number of points between each high symmetry point.
+            This is also only required for unfolded calculations. This number should be 
             known by the user, as it was used to generate the KPOINTS file.
     """
 
-    def __init__(self, folder, projected=False, hse=False, unfold=False, spin='up', kpath=None, n=None, M=None, high_symm_points=None):
+    def __init__(self, folder, projected=False, unfold=False, spin='up', kpath=None, n=None, M=None, high_symm_points=None, bandgap=False, printbg=True):
         """
         Initialize parameters upon the generation of this class
 
         Parameters:
             folder (str): This is the folder that contains the VASP files
-            projected (bool): Determined wheter of not to parte the projected
-                eigenvalues from the vasprun.xml file. Making this true
+            projected (bool): Determines whether of not to parse the projected
+                eigenvalues from the PROCAR file. Making this true
                 increases the computational time, so only use if a projected
                 band structure is required.
-            hse (bool): Determines if the KPOINTS file is in the form of HSE
-                or not. Only make true if the band structure was calculated
-                using a hybrid functional.
             spin (str): Choose which spin direction to parse. ('up' or 'down')
+            kpath (str): High symmetry k-point path of band structure calculation
+                Due to the nature of the KPOINTS file for unfolded calculations this
+                information is a required input for proper labeling of the figure
+                for unfolded calculations. This information is extracted from the KPOINTS
+                files for non-unfolded calculations. (G is automaticall converted to \\Gamma)
+            n (int): Number of points between each high symmetry point.
+                This is also only required for unfolded calculations. This number should be 
+                known by the user, as it was used to generate the KPOINTS file.
         """
 
+        self.bandgap = bandgap
+        self.printbg = printbg
+        self.bg = None
         self.eigenval = Eigenval(os.path.join(folder, 'EIGENVAL'))
         self.efermi = float(os.popen(f'grep E-fermi {os.path.join(folder, "OUTCAR")}').read().split()[2])
         self.poscar = Poscar.from_file(
@@ -79,11 +85,26 @@ class Band:
         else:
             self.lsorbit = False
 
+        if 'ISPIN' in self.incar:
+            if self.incar['ISPIN'] == 2:
+                self.ispin = True
+            else:
+                self.ispin = False
+        else:
+            self.ispin = False
+
+
+        self.kpoints_file = Kpoints.from_file(os.path.join(folder, 'KPOINTS'))
+
         self.wavecar = os.path.join(folder, 'WAVECAR')
         self.projected = projected
         self.forbitals = False
-        self.hse = hse
+        self.hse = str(self.kpoints_file._style) != 'Line_mode'
         self.unfold = unfold
+
+        if self.hse and self.unfold:
+            self.hse = False
+
         self.kpath = kpath
         self.n = n
         self.M = M
@@ -145,8 +166,24 @@ class Band:
             self.pre_loaded_projections = os.path.isfile(os.path.join(folder, 'projected_eigenvalues.npy'))
             self.projected_eigenvalues = self._load_projected_bands()
 
-        if not hse:
-            self.kpoints_file = Kpoints.from_file(os.path.join(folder, 'KPOINTS'))
+    def _get_bandgap(self, eigenvalues):
+        if np.sum(np.diff(np.sign(eigenvalues[:,:,0])) != 0) == 0:
+            occupied = eigenvalues[np.where(eigenvalues[:,:,-1] > 1e-8)]
+            unoccupied = eigenvalues[np.where(eigenvalues[:,:,-1] < 1e-8)]
+
+            vbm = np.max(occupied[:,0])
+            cbm = np.min(unoccupied[:,0])
+
+            bg = cbm - vbm
+        else:
+            bg = 0
+
+        if self.printbg:
+            print(f'Bandgap = {np.round(bg, 3)} eV')
+
+        self.bg = bg
+
+
 
     def _load_bands(self):
         """
@@ -159,30 +196,85 @@ class Band:
                 the eigenvalues for each band
         """
 
-        spin = self.spin_dict[self.spin]
+        if self.spin == 'up':
+            spin = 0
+        if self.spin == 'down':
+            spin = 1
 
         if self.pre_loaded_bands:
             with open(os.path.join(self.folder, 'eigenvalues.npy'), 'rb') as eigenvals:
                 band_data = np.load(eigenvals)
 
-            eigenvalues = band_data[:,:,0]
-            kpoints = band_data[0,:,1:]
+            if self.ispin and not self.lsorbit:
+                eigenvalues = band_data[:,:,[0,2]]
+                kpoints = band_data[0,:,4:]
+                if self.bandgap:
+                    eigenvalues_up = band_data[:,:,[0,1]]
+                    eigenvalues_down = band_data[:,:,[2,3]]
+                    eigenvalues_bg = np.vstack([eigenvalues_up, eigenvalues_down])
+            else:
+                eigenvalues = band_data[:,:,0]
+                kpoints = band_data[0,:,2:]
+                if self.bandgap:
+                    eigenvalues_bg = band_data[:,:,[0,1]]
+            
+            if self.bandgap:
+                self._get_bandgap(eigenvalues=eigenvalues_bg)
         else:
-            eigenvalues = np.transpose(self.eigenval.eigenvalues[spin][:,:,0]) - self.efermi
+            if len(self.eigenval.eigenvalues.keys()) > 1:
+                eigenvalues_up = np.transpose(self.eigenval.eigenvalues[Spin.up], axes=(1,0,2))
+                eigenvalues_down = np.transpose(self.eigenval.eigenvalues[Spin.down], axes=(1,0,2))
+                eigenvalues_up[:,:,0] = eigenvalues_up[:,:,0] - self.efermi
+                eigenvalues_down[:,:,0] = eigenvalues_down[:,:,0] - self.efermi
+                eigenvalues = np.concatenate(
+                    [eigenvalues_up, eigenvalues_down],
+                    axis=2
+                )
+                print(eigenvalues.shape)
+                #  eigenvalues = np.concatenate(
+                    #  [eigenvalues_up[:,:,np.newaxis], eigenvalues_down[:,:,np.newaxis]],
+                    #  axis=2
+                #  )
+                if self.bandgap:
+                    eigenvalues_bg = np.vstack([eigenvalues_up, eigenvalues_down])
+            else:
+                eigenvalues = np.transpose(self.eigenval.eigenvalues[Spin.up], axes=(1,0,2))
+                eigenvalues[:,:,0] = eigenvalues[:,:,0] - self.efermi
+                if self.bandgap:
+                    eigenvalues_bg = eigenvalues
+
             kpoints = np.array(self.eigenval.kpoints)
 
             if self.hse:
-                kpoints_band = self.n * (len(self.kpath) - 1)
-                eigenvalues = eigenvalues[-kpoints_band:]
-                kpoints = kpoints[-kpoints_band:]
+                kpoint_weights = np.array(self.eigenval.kpoints_weights)
+                zero_weight = np.where(kpoint_weights == 0)[0]
+                eigenvalues = eigenvalues[:,zero_weight]
+                if self.bandgap:
+                    eigenvalues_bg = eigenvalues_bg[:, zero_weight]
+                kpoints = kpoints[zero_weight]
 
+            if self.bandgap:
+                self._get_bandgap(eigenvalues=eigenvalues_bg)
+
+            #  if len(self.eigenval.eigenvalues.keys()) > 1:
             band_data = np.append(
-                eigenvalues[:,:,np.newaxis],
+                eigenvalues,
                 np.tile(kpoints, (eigenvalues.shape[0],1,1)),
                 axis=2,
             )
+            #  else:
+                #  band_data = np.append(
+                    #  eigenvalues[:,:,np.newaxis],
+                    #  np.tile(kpoints, (eigenvalues.shape[0],1,1)),
+                    #  axis=2,
+                #  )
+
             np.save(os.path.join(self.folder, 'eigenvalues.npy'), band_data)
 
+        if len(self.eigenval.eigenvalues.keys()) > 1:
+            eigenvalues = eigenvalues[:,:,spin]
+
+
         return eigenvalues, kpoints
 
     
@@ -244,19 +336,32 @@ class Band:
         else:
             parser = ProcarParser()
             parser.readFile(os.path.join(self.folder, 'PROCAR_repaired'))
-            projected_eigenvalues = np.transpose(parser.spd[:,:,spin,:-1, 1:-1], axes=(1,0,2,3))
-            projected_eigenvalues = projected_eigenvalues 
-            #  / np.sum(np.sum(projected_eigenvalues, axis=3),axis=2)[:,:,np.newaxis,np.newaxis]
+            if self.ispin and not self.lsorbit and np.sum(self.poscar.natoms) == 1:
+                shape = int(parser.spd.shape[1] / 2)
+                projected_eigenvalues_up = np.transpose(parser.spd[:,:shape,0,:,1:-1], axes=(1,0,2,3))
+                projected_eigenvalues_down = np.transpose(parser.spd[:,shape:,0,:,1:-1], axes=(1,0,2,3))
+                projected_eigenvalues = np.concatenate(
+                    [projected_eigenvalues_up[:,:,:,:,np.newaxis], projected_eigenvalues_down[:,:,:,:,np.newaxis]],
+                    axis=4
+                )
+                projected_eigenvalues = np.transpose(projected_eigenvalues, axes=(0,1,4,2,3))
+            else:
+                projected_eigenvalues = np.transpose(parser.spd[:,:,:,:-1, 1:-1], axes=(1,0,2,3,4))
+
             np.save(os.path.join(self.folder, 'projected_eigenvalues.npy'), projected_eigenvalues)
 
+
+        projected_eigenvalues = projected_eigenvalues[:,:,spin,:,:]
+
         if self.hse:
-            kpoints_band = self.n * (len(self.kpath) - 1)
-            projected_eigenvalues = projected_eigenvalues[-kpoints_band:]
+            kpoint_weights = np.array(self.eigenval.kpoints_weights)
+            zero_weight = np.where(kpoint_weights == 0)[0]
+            projected_eigenvalues = projected_eigenvalues[:,zero_weight]
 
         if projected_eigenvalues.shape[-1] == 16:
             self.forbitals = True
 
-        #  projected_eigenvalues = np.square(projected_eigenvalues)
+        projected_eigenvalues = np.square(projected_eigenvalues)
 
         return projected_eigenvalues
 
@@ -438,14 +543,14 @@ class Band:
                 np.sum(element_orbitals[:,:,:,ind], axis=3) for ind in spd_indices
             ]), axes=(1,2,3,0))
 
-            norm_term = np.sum(element_spd, axis=(2,3))[:,:,np.newaxis, np.newaxis]
-            element_spd = np.divide(element_spd, norm_term, out=np.zeros_like(element_spd), where=norm_term!=0)
+            #  norm_term = np.sum(element_spd, axis=(2,3))[:,:,np.newaxis, np.newaxis]
+            #  element_spd = np.divide(element_spd, norm_term, out=np.zeros_like(element_spd), where=norm_term!=0)
 
             return element_spd
         else:
             element_array = np.sum(element_orbitals, axis=3)
-            norm_term = np.sum(element_array, axis=2)[:,:,np.newaxis]
-            element_array = np.divide(element_array, norm_term, out=np.zeros_like(element_array), where=norm_term!=0)
+            #  norm_term = np.sum(element_array, axis=2)[:,:,np.newaxis]
+            #  element_array = np.divide(element_array, norm_term, out=np.zeros_like(element_array), where=norm_term!=0)
 
             return element_array
 
@@ -462,7 +567,7 @@ class Band:
     def _get_kticks(self, ax):
         """
         This function extracts the kpoint labels and index locations for a regular
-        band structure calculation (non HSE).
+        band structure calculation (non unfolded).
 
         Parameters:
             ax (matplotlib.pyplot.axis): Axis to append the tick labels
@@ -491,14 +596,26 @@ class Band:
         ax.set_xticks(kpoints_index)
         ax.set_xticklabels(kpts_labels)
 
-    def _get_kticks_hse(self, ax, kpath, n):
-        kpoints_index = [(i*n) - 1 for i in range(len(kpath))
-                         if 0 < i < len(kpath)-1]
-        kpoints_index.append(n*(len(kpath) - 1)-1)
-        kpoints_index.insert(0, 0)
+    def _get_kticks_hse(self, ax, kpath):
+        structure = self.poscar.structure
+        kpath_obj = HighSymmKpath(structure)
+        kpath_labels = np.array(list(kpath_obj._kpath['kpoints'].keys()))
+        kpath_coords = np.array(list(kpath_obj._kpath['kpoints'].values()))
+        index = np.where((self.kpoints[:, None] == kpath_coords).all(-1).any(-1) == True)[0]
+        index = [index[0]] + [index[i] for i in range(1,len(index)-1) if i % 2] + [index[-1]]
+        kpoints_in_band = self.kpoints[index]
+
+        label_index = []
+        for i in range(kpoints_in_band.shape[0]):
+            for j in range(kpath_coords.shape[0]):
+                if (kpoints_in_band[i] == kpath_coords[j]).all():
+                    label_index.append(j)
+
+        kpoints_index = index
+        kpath = kpath_labels[label_index]
         kpoints_index = ax.lines[0].get_xdata()[kpoints_index]
 
-        kpath = [f'${k}$' if k != 'G' else '$\\Gamma$' for k in kpath.upper().strip()]
+        kpath = [f'${k}$' if k != 'G' else '$\\Gamma$' for k in kpath]
 
         for k in kpoints_index:
             ax.axvline(x=k, color='black', alpha=0.7, linewidth=0.5)
@@ -607,7 +724,7 @@ class Band:
             )
 
         if self.hse:
-            self._get_kticks_hse(ax=ax, kpath=self.kpath, n=self.n)
+            self._get_kticks_hse(ax=ax, kpath=self.kpath)
         elif self.unfold:
             self._get_kticks_unfold(ax=ax, wave_vectors=wave_vectors)
         else:
@@ -684,178 +801,6 @@ class Band:
             s=s,
             zorder=100,
         )
-
-    #  def _pie_scatter(self, ax, x, y, s, fractions, colors):
-        #  #  patches = []
-        #  fractions = np.c_[np.zeros(fractions.shape[0]), fractions]
-        #  for i in range(len(x)):
-            #  indv_fraction = fractions[i]
-            #  indv_color = colors
-            #  if not np.isclose(np.sum(indv_fraction), 1, atol=0.01):
-                #  indv_fraction = np.append(indv_fraction, 1 - np.sum(indv_fraction))
-                #  indv_color = np.append(indv_color, 'black')
-            #  transform = (fig.dpi_scale_trans + transforms.ScaledTranslation(x[i], y[i], ax.transData))
-            #  wedges = [
-                #  Wedge(
-                    #  (0,0),
-                    #  np.sqrt(s[i] / 1.5) / 72,
-                    #  360 * np.sum(indv_fraction[:j+1]),
-                    #  360 * np.sum(indv_fraction[:j+2]),
-                    #  color=indv_color[j],
-                    #  zorder=150,
-                    #  ec=None,
-                    #  #  transform=transform,
-                #  ) for j in range(len(indv_color))
-            #  ]
-            #  #  patches.extend(wedges)
-#
-            #  collection = PatchCollection(wedges, transform=transform, match_original=True)
-            #  ax.add_collection(collection)
-        #  #  [ax.add_patch(patch) for patch in patches]
-        #  #  plt.axis('equal', adjustable='datalim')
-        #  #  ax.set_xlim(-0.5,0.5)
-#
-    #  def _plot_projected_general_unfold(self, ax, projected_data, colors, scale_factor=5, erange=[-6,6], display_order=None, linewidth=0.75, band_color='black'):
-        #  """
-        #  This is a general method for plotting projected data
-#
-        #  Parameters:
-            #  scale_factor (float): Factor to scale weights. This changes the size of the
-                #  points in the scatter plot
-            #  color_dict (dict[str][str]): This option allow the colors of each orbital
-                #  specified. Should be in the form of:
-                #  {'orbital index': <color>, 'orbital index': <color>, ...}
-            #  legend (bool): Determines if the legend should be included or not.
-            #  linewidth (float): Line width of the plain band structure plotted in the background
-            #  band_color (string): Color of the plain band structure
-        #  """
-        #  #  self.plot_plain(ax=ax, linewidth=linewidth, scale_factor=scale_factor, color=band_color, erange=erange)
-        #  scale_factor = scale_factor + (scale_factor * 0.5)
-        #  bands_in_plot = self._filter_bands(erange=erange)
-        #  K_indices = np.array(self.K_indices[0], dtype=int)
-        #  projected_data = projected_data[bands_in_plot]
-        #  projected_data = projected_data[:, K_indices, :]
-        #  wave_vectors = self._get_k_distance()
-        #  eigenvalues = self.eigenvalues[bands_in_plot]
-        #  spectral_weights = self.spectral_weights[bands_in_plot]
-#
-        #  projected_data_ravel = np.square(np.ravel(projected_data))
-        #  wave_vectors_tile = np.tile(
-            #  np.repeat(wave_vectors, projected_data.shape[-1]), projected_data.shape[0]
-        #  )
-        #  eigenvalues_ravel = np.repeat(np.ravel(eigenvalues), projected_data.shape[-1])
-        #  colors_tile = np.tile(colors, np.prod(projected_data.shape[:-1]))
-        #  spectral_weights_ravel = np.repeat(np.ravel(spectral_weights), projected_data.shape[-1])
-#
-        #  if display_order is None:
-            #  pass
-        #  else:
-            #  sort_index = np.argsort(projected_data_ravel)
-#
-            #  if display_order == 'all':
-                #  sort_index = sort_index[::-1]
-#
-            #  wave_vectors_tile = wave_vectors_tile[sort_index]
-            #  eigenvalues_ravel = eigenvalues_ravel[sort_index]
-            #  colors_tile = colors_tile[sort_index]
-            #  projected_data_ravel = projected_data_ravel[sort_index]
-            #  spectral_weights_ravel = spectral_weights_ravel[sort_index]
-#
-        #  ax.scatter(
-            #  wave_vectors_tile,
-            #  eigenvalues_ravel,
-            #  c=colors_tile,
-            #  s=scale_factor * projected_data_ravel * spectral_weights_ravel,
-            #  zorder=100,
-        #  )
-#
-#
-        #  if self.hse:
-            #  self._get_kticks_hse(ax=ax, kpath=self.kpath, n=self.n)
-        #  elif self.unfold:
-            #  self._get_kticks_unfold(ax=ax, wave_vectors=wave_vectors)
-        #  else:
-            #  self._get_kticks(ax=ax)
-#
-        #  ax.set_xlim(0, np.max(wave_vectors))
-#
-    #  def _plot_projected_general_unfold_old(self, ax, projected_data, colors, scale_factor=5, erange=[-6,6], display_order=None, linewidth=0.75, band_color='black'):
-        #  """
-        #  This is a general method for plotting projected data
-#
-        #  Parameters:
-            #  scale_factor (float): Factor to scale weights. This changes the size of the
-                #  points in the scatter plot
-            #  color_dict (dict[str][str]): This option allow the colors of each orbital
-                #  specified. Should be in the form of:
-                #  {'orbital index': <color>, 'orbital index': <color>, ...}
-            #  legend (bool): Determines if the legend should be included or not.
-            #  linewidth (float): Line width of the plain band structure plotted in the background
-            #  band_color (string): Color of the plain band structure
-        #  """
-        #  #  self.plot_plain(ax=ax, linewidth=linewidth, scale_factor=0.1, color=band_color, erange=erange)
-        #  scale_factor = scale_factor + (scale_factor * 0.5)
-        #  bands_in_plot = self._filter_bands(erange=erange)
-        #  K_indices = np.array(self.K_indices[0], dtype=int)
-        #  projected_data = projected_data[bands_in_plot]
-        #  projected_data = projected_data[:, K_indices, :]
-        #  wave_vectors = self._get_k_distance()
-        #  eigenvalues = self.eigenvalues[bands_in_plot]
-        #  spectral_weights = self.spectral_weights[bands_in_plot]
-#
-        #  if display_order is None or display_order == 'all':
-            #  projected_data_reshape = np.reshape(
-                #  projected_data, (np.prod(projected_data.shape[:2]), np.prod(projected_data.shape[2:]))
-            #  )
-#
-            #  wave_vectors_tile = np.tile(wave_vectors, projected_data.shape[0])
-            #  eigenvalues_ravel = np.ravel(eigenvalues)
-            #  spectral_weights_ravel = np.ravel(spectral_weights)
-#
-            #  self._pie_scatter(
-                #  ax=ax,
-                #  x=wave_vectors_tile,
-                #  y=eigenvalues_ravel,
-                #  colors=colors,
-                #  s=scale_factor*spectral_weights_ravel,
-                #  fractions=projected_data_reshape,
-            #  )
-        #  elif display_order == 'dominant':
-            #  projected_data_ravel = np.square(np.ravel(projected_data))
-            #  wave_vectors_tile = np.tile(
-                #  np.repeat(wave_vectors, projected_data.shape[-1]), projected_data.shape[0]
-            #  )
-            #  eigenvalues_ravel = np.repeat(np.ravel(eigenvalues), projected_data.shape[-1])
-            #  colors_tile = np.tile(colors, np.prod(projected_data.shape[:-1]))
-            #  spectral_weights_ravel = np.repeat(np.ravel(spectral_weights), projected_data.shape[-1])
-#
-            #  sort_index = np.argsort(projected_data_ravel)
-            #  sort_index = sort_index[::-1]
-#
-            #  wave_vectors_tile = wave_vectors_tile[sort_index]
-            #  eigenvalues_ravel = eigenvalues_ravel[sort_index]
-            #  colors_tile = colors_tile[sort_index]
-            #  projected_data_ravel = projected_data_ravel[sort_index]
-            #  spectral_weights_ravel = spectral_weights_ravel[sort_index]
-#
-            #  ax.scatter(
-                #  wave_vectors_tile,
-                #  eigenvalues_ravel,
-                #  c=colors_tile,
-                #  ec=None,
-                #  s=scale_factor * projected_data_ravel * spectral_weights_ravel,
-                #  zorder=100,
-            #  )
-#
-#
-        #  if self.hse:
-            #  self._get_kticks_hse(ax=ax, kpath=self.kpath, n=self.n)
-        #  elif self.unfold:
-            #  self._get_kticks_unfold(ax=ax, wave_vectors=wave_vectors)
-        #  else:
-            #  self._get_kticks(ax=ax)
-#
-        #  ax.set_xlim(0, np.max(wave_vectors))
         
     def plot_orbitals(self, ax, orbitals, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
         """
@@ -910,7 +855,8 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(ax, names=[self.orbital_labels[i] for i in orbitals], colors=colors)
+        if legend:
+            self._add_legend(ax, names=[self.orbital_labels[i] for i in orbitals], colors=colors)
 
 
     def plot_spd(self, ax, scale_factor=5, orbitals='spd', erange=[-6,6], display_order=None, color_dict=None, legend=True, linewidth=0.75, band_color='black'):
@@ -942,9 +888,6 @@ class Band:
                 3: self.color_dict[4],
             }
 
-        if self.forbitals:
-            orbitals = orbitals + 'f'
-
         colors = np.array([color_dict[self.spd_relations[i]] for i in orbitals])
 
         projected_data = self._sum_spd(spd=orbitals)
@@ -960,7 +903,8 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(ax, names=[i for i in orbitals], colors=colors)
+        if legend:
+            self._add_legend(ax, names=[i for i in orbitals], colors=colors)
 
 
 
@@ -996,7 +940,8 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(ax, names=atoms, colors=colors)
+        if legend:
+            self._add_legend(ax, names=atoms, colors=colors)
 
 
     def plot_atom_orbitals(self, ax, atom_orbital_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
@@ -1048,11 +993,12 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(
-            ax,
-            names=[f'{i[0]}, {i[1]}' for i in zip(atom_indices, orbital_symbols_long)],
-            colors=colors
-        )
+        if legend:
+            self._add_legend(
+                ax,
+                names=[f'{i[0]}({i[1]})' for i in zip(atom_indices, orbital_symbols_long)],
+                colors=colors
+            )
 
     def plot_atom_spd(self, ax, atom_spd_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
         """
@@ -1104,11 +1050,12 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(
-            ax,
-            names=[f'{i[0]}, {i[1]}' for i in zip(atom_indices, orbital_symbols_long)],
-            colors=colors
-        )
+        if legend:
+            self._add_legend(
+                ax,
+                names=[f'{i[0]}({i[1]})' for i in zip(atom_indices, orbital_symbols_long)],
+                colors=colors
+            )
 
 
 
@@ -1144,7 +1091,8 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(ax, names=elements, colors=colors)
+        if legend:
+            self._add_legend(ax, names=elements, colors=colors)
 
 
     def plot_element_orbitals(self, ax, element_orbital_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
@@ -1192,11 +1140,12 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(
-            ax,
-            names=[f'{i[0]}, {i[1]}' for i in zip(element_symbols_long, orbital_symbols_long)],
-            colors=colors
-        )
+        if legend:
+            self._add_legend(
+                ax,
+                names=[f'{i[0]}({i[1]})' for i in zip(element_symbols_long, orbital_symbols_long)],
+                colors=colors
+            )
 
     def plot_element_spd(self, ax, element_spd_dict, scale_factor=5, erange=[-6,6], display_order=None, color_list=None, legend=True, linewidth=0.75, band_color='black'):
         """
@@ -1250,11 +1199,12 @@ class Band:
             band_color=band_color
         )
 
-        self._add_legend(
-            ax,
-            names=[f'{i[0]}, {i[1]}' for i in zip(element_symbols_long, orbital_symbols_long)],
-            colors=colors
-        )
+        if legend:
+            self._add_legend(
+                ax,
+                names=[f'{i[0]}({i[1]})' for i in zip(element_symbols_long, orbital_symbols_long)],
+                colors=colors
+            )
 
 
 if __name__ == "__main__":
